using BankBusinessLogic.BusinessLogics;
using BankContracts.BusinessLogicsContracts;
using BankContracts.StoragesContracts;
using BankDatabaseImplement.Implements;
using BankContracts.BindingModels;
using BankOperatorApp;
using BankBusinessLogic.OfficePackage.Implements;
using BankBusinessLogic.OfficePackage;
using BankBusinessLogic.MailWorker;

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.
builder.Services.AddControllersWithViews();

builder.Services.AddTransient<ICurrencyPurchaseStorage, CurrencyPurchaseStorage>();
builder.Services.AddTransient<ICurrencyStorage, CurrencyStorage>();
builder.Services.AddTransient<ICreditProgramStorage, CreditProgramStorage>();
builder.Services.AddTransient<IBankOperatorStorage, BankOperatorStorage>();

builder.Services.AddTransient<IDealStorage, DealStorage>();
builder.Services.AddTransient<ITransferStorage, TransferStorage>();
builder.Services.AddTransient<IPaymentStorage, PaymentStorage>();

builder.Services.AddTransient<AbstractSaveToWord, SaveToWord>();
builder.Services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
builder.Services.AddTransient<AbstractSaveToPdf, SaveToPdf>();
builder.Services.AddSingleton<AbstractMailWorker, MailKitWorker>();


builder.Services.AddTransient<ICurrencyPurchaseLogic, CurrencyPurchaseLogic>();
builder.Services.AddTransient<ICurrencyLogic, CurrencyLogic>();
builder.Services.AddTransient<ICreditProgramLogic, CreditProgramLogic>();
builder.Services.AddTransient<IBankOperatorLogic, BankOperatorLogic>();
builder.Services.AddTransient<IReportLogic, ReportLogic>();
builder.Services.AddTransient<IDealLogic, DealLogic>();

builder.Services.AddControllersWithViews();

var app = builder.Build();
try
{
    var mailSender = app.Services.GetService<AbstractMailWorker>();
    mailSender?.MailConfig(new MailConfigBindingModel
    {
        MailLogin = builder.Configuration["MailLogin"] ?? string.Empty,
        MailPassword = builder.Configuration["MailPassword"] ?? string.Empty,
        SmtpClientHost = builder.Configuration["SmtpClientHost"] ?? string.Empty,
        SmtpClientPort = Convert.ToInt32(builder.Configuration["SmtpClientPort"]),
        PopHost = builder.Configuration["PopHost"] ?? string.Empty,
        PopPort = Convert.ToInt32(builder.Configuration["PopPort"])
    });
}
catch (Exception ex)
{
    var logger = app.Services.GetService<ILogger>();
    logger?.LogError(ex, "   ");
}

// Configure the HTTP request pipeline.
if (!app.Environment.IsDevelopment())
{
    app.UseExceptionHandler("/Home/Error");
    // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
    app.UseHsts();
}

app.UseHttpsRedirection();
app.UseStaticFiles();

app.UseRouting();

app.UseAuthorization();

app.MapControllerRoute(
    name: "default",
    pattern: "{controller=Home}/{action=Index}/{id?}");

app.Run();
